//Project: CD ROM plotter
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 0.1
//Creation date: 11.06.2016
//compile with: gcc Stepper-Raspberry-Pi.c -lm -lwiringPi -o Stepper-Raspberry-Pi
//Start with sudo ./Stepper-Raspberry-Pi

#include <stdio.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <dirent.h>
#include <math.h>
#include <wiringPi.h>
#include <unistd.h>


#define X_STEPPER01  5
#define X_STEPPER02  6

#define X_STEPPER03 10
#define X_STEPPER04 11



int  MaxRows = 24;
int  MaxCols = 80;
int  MessageX = 1;
int  MessageY = 24;

int StepX = 0;
int StepY = 0;
double StepsPermmX = 250.0 / 35.0;
double StepsPermmY = 250.0 / 35.0;


//+++++++++++++++++++++++ Start gotoxy ++++++++++++++++++++++++++
//Thanks to 'Stack Overflow', found on http://www.daniweb.com/software-development/c/code/216326
int gotoxy(int x, int y) {
  char essq[100]; // String variable to hold the escape sequence
  char xstr[100]; // Strings to hold the x and y coordinates
  char ystr[100]; // Escape sequences must be built with characters

  //Convert the screen coordinates to strings.
  sprintf(xstr, "%d", x);
  sprintf(ystr, "%d", y);

  //Build the escape sequence (vertical move).
  essq[0] = '\0';
  strcat(essq, "\033[");
  strcat(essq, ystr);

  //Described in man terminfo as vpa=\E[%p1%dd. Vertical position absolute.
  strcat(essq, "d");

  //Horizontal move. Horizontal position absolute
  strcat(essq, "\033[");
  strcat(essq, xstr);
  // Described in man terminfo as hpa=\E[%p1%dG
  strcat(essq, "G");

  //Execute the escape sequence. This will move the cursor to x, y
  printf("%s", essq);
  return 0;
}
//------------------------ End gotoxy ----------------------------------

//+++++++++++++++++++++++ Start clrscr ++++++++++++++++++++++++++
void clrscr(int StartRow, int EndRow) {
  int i, i2;

  if (EndRow < StartRow){
    i = EndRow;
    EndRow = StartRow;
    StartRow = i;
  }
  gotoxy(1, StartRow);
  for (i = 0; i <= EndRow - StartRow; i++){
    for(i2 = 0; i2 < MaxCols; i2++){
      printf(" ");
    }
    printf("\n");
  }
}
//----------------------- End clrscr ----------------------------

//+++++++++++++++++++++++ Start kbhit ++++++++++++++++++++++++++++++++++
//Thanks to Undertech Blog, http://www.undertec.de/blog/2009/05/kbhit_und_getch_fur_linux.html
int kbhit(void) {

   struct termios term, oterm;
   int fd = 0;
   int c = 0;

   tcgetattr(fd, &oterm);
   memcpy(&term, &oterm, sizeof(term));
   term.c_lflag = term.c_lflag & (!ICANON);
   term.c_cc[VMIN] = 0;
   term.c_cc[VTIME] = 1;
   tcsetattr(fd, TCSANOW, &term);
   c = getchar();
   tcsetattr(fd, TCSANOW, &oterm);
   if (c != -1)
   ungetc(c, stdin);

   return ((c != -1) ? 1 : 0);

}
//------------------------ End kbhit -----------------------------------

//+++++++++++++++++++++++ Start getch ++++++++++++++++++++++++++++++++++
//Thanks to Undertech Blog, http://www.undertec.de/blog/2009/05/kbhit_und_getch_fur_linux.html
int getch(){
   static int ch = -1, fd = 0;
   struct termios new, old;

   fd = fileno(stdin);
   tcgetattr(fd, &old);
   new = old;
   new.c_lflag &= ~(ICANON|ECHO);
   tcsetattr(fd, TCSANOW, &new);
   ch = getchar();
   tcsetattr(fd, TCSANOW, &old);

//   printf("ch=%d ", ch);

   return ch;
}
//------------------------ End getch -----------------------------------

//++++++++++++++++++++++ Start MessageText +++++++++++++++++++++++++++++
void MessageText(char *message, int x, int y, int alignment){
  int i;
  char TextLine[300];

  clrscr(y, y);
  gotoxy (x, y);

  TextLine[0] = '\0';
  if(alignment == 1){
    for(i=0; i < (MaxCols - strlen(message)) / 2 ; i++){
      strcat(TextLine, " ");
    }
  }
  strcat(TextLine, message);

  printf("%s\n", TextLine);
}
//-------------------------- End MessageText ---------------------------

//++++++++++++++++++++++ Start PrintRow ++++++++++++++++++++++++++++++++
void PrintRow(char character, int y){
  int i;
  gotoxy (1, y);
  for(i=0; i<MaxCols;i++){
    printf("%c", character);
  }
}
//-------------------------- End PrintRow ------------------------------

//+++++++++++++++++++++++++ ErrorText +++++++++++++++++++++++++++++
void ErrorText(char *message){
  clrscr(MessageY + 2, MessageY + 2);
  gotoxy (1, MessageY + 2);
  printf("Last error: %s", message);
}
//----------------------------- ErrorText ---------------------------

//+++++++++++++++++++++++++ PrintMenue_01 ++++++++++++++++++++++++++++++
void PrintMenue_01(long MoveLength){
  char TextLine[300];

   clrscr(1, MessageY-2);
   MessageText("*** Main menu plotter ***", 1, 1, 1);
   sprintf(TextLine, "M            - toggle move length, current value = %ld step(s)", MoveLength);
   MessageText(TextLine, 10, 3, 0);
   MessageText("Cursor right - move motor in positive X direction", 10, 4, 0);
   MessageText("Cursor left  - move motor in negative X direction", 10, 5, 0);




   MessageText("Esc          - leave program", 10, 16, 0);

}
//------------------------- PrintMenue_01 ------------------------------



//++++++++++++++++++++++++++++++ MakeStepX ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void MakeStepX(int direction, long stepPause){
  StepX += direction;

  if(StepX > 3){
    StepX = 0;
  }
  if(StepX < 0){
    StepX = 3;
  }


  if(StepX == 0){
    digitalWrite(X_STEPPER01, 1);
    digitalWrite(X_STEPPER02, 0);
    digitalWrite(X_STEPPER03, 0);
    digitalWrite(X_STEPPER04, 0);
  }
  if(StepX == 1){
    digitalWrite(X_STEPPER01, 0);
    digitalWrite(X_STEPPER02, 0);
    digitalWrite(X_STEPPER03, 1);
    digitalWrite(X_STEPPER04, 0);
  }
  if(StepX == 2){
    digitalWrite(X_STEPPER01, 0);
    digitalWrite(X_STEPPER02, 1);
    digitalWrite(X_STEPPER03, 0);
    digitalWrite(X_STEPPER04, 0);
  }
  if(StepX == 3){
    digitalWrite(X_STEPPER01, 0);
    digitalWrite(X_STEPPER02, 0);
    digitalWrite(X_STEPPER03, 0);
    digitalWrite(X_STEPPER04, 1);
  }

  usleep(stepPause);
}


//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  int KeyHit = 0;
  int KeyCode[5];
  char FileInfo[3];
  char FileName[200] = "";
  char FullFileName[200] = "";
  char FileNameOld[200] = "";
  struct winsize terminal;
  double PicHeight = 0.0;
  long MoveLength = 1;
  long OldMoveLength = 200;
  int i;
  int SingleKey=0;
  long stepPause = 3000;
  char *pEnd;
  FILE *PlotFile;


  FileInfo[2]='\0';

  strcpy(FileName, "noFiLE");


  if (wiringPiSetup () == -1){
    printf("Could not run wiringPiSetup!");
    exit(1);
  }



  pinMode (X_STEPPER01, OUTPUT);
  pinMode (X_STEPPER02, OUTPUT);
  pinMode (X_STEPPER03, OUTPUT);
  pinMode (X_STEPPER04, OUTPUT);

  digitalWrite(X_STEPPER01, 1);
  digitalWrite(X_STEPPER02, 0);
  digitalWrite(X_STEPPER03, 0);
  digitalWrite(X_STEPPER04, 0);


  if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &terminal)<0){
    printf("Can't get size of terminal window");
  }
  else{
    MaxRows = terminal.ws_row;
    MaxCols = terminal.ws_col;
    MessageY = MaxRows-3;
  }

  clrscr(1, MaxRows);
  PrintRow('-', MessageY - 1);
  PrintMenue_01(MoveLength);


  while (1){
    MessageText("Waiting for key press.", MessageX, MessageY, 0);

    i = 0;
    SingleKey = 1;
    KeyCode[0] = 0;
    KeyCode[1] = 0;
    KeyCode[2] = 0;
    KeyCode[3] = 0;
    KeyCode[4] = 0;
    KeyHit = 0;
    while (kbhit()){
      KeyHit = getch();
      KeyCode[i] = KeyHit;
      i++;
      if(i == 5){
        i = 0;
      }
      if(i > 1){
        SingleKey = 0;
      }
    }
    if(SingleKey == 0){
      KeyHit = 0;
    }


    //Move X-axis
    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 68 && KeyCode[3] == 0 && KeyCode[4] == 0){
      for(i = 0; i < MoveLength; i++){
        MakeStepX(-1, stepPause);
      }
    }

    if(KeyCode[0] == 27 && KeyCode[1] == 91 && KeyCode[2] == 67 && KeyCode[3] == 0 && KeyCode[4] == 0){
      for(i = 0; i < MoveLength; i++){
        MakeStepX(1, stepPause);
      }
    }


    if(KeyHit == 'm'){
      MoveLength *= 10;
      if(MoveLength > 1000){
        MoveLength = 1;
      }
      PrintMenue_01(MoveLength);
    }



    if(KeyHit == 27){
      clrscr(MessageY + 1, MessageY + 1);
      MessageText("Exit program (y/n)?", MessageX, MessageY + 1, 0);
      while(KeyHit != 'y' && KeyHit != 'n'){
        KeyHit = getch();
        if(KeyHit == 'y'){
          digitalWrite(X_STEPPER01, 0);
          digitalWrite(X_STEPPER02, 0);
          digitalWrite(X_STEPPER03, 0);
          digitalWrite(X_STEPPER04, 0);
          exit(0);
        }
      }
      clrscr(MessageY + 1, MessageY + 1);
    }
  }

  return 0;
}
