//Printer motor
//Source & info at: www.HomoFaciens.de/technics-base-circuits-encoder-disc_en_navion.htm
//Sensor disc with 2.2k, 560 and 0 Ohms

#include <util/delay.h>

int stepStatus = 0;


int dutyCycle = 0;
signed long setPoint = 0;
signed long actualPoint = 0;
byte readByte = 0;
int sensorStatus = 0;
int sensorStatusOld = 0;
unsigned long millisStart = 0;
unsigned long millisNow = 0;
int stepTemp = 0;
int stepTempOld = 0;
int step = 0;
int stepOld = 0;


#define SENSOR  A0

#define MOTOR_DIRECTION 5
#define MOTOR_PWM 6


#define P_FRACTION 1.0     //Proportional factor of control loop 0.001 - 10.0 (1.0)
#define STEP_MARGIN 1L     //10 - 1000 (1)

#define MIN_DUTYCYCLE 200   //0 - 255 (125)
#define MAX_DUTYCYCLE 250  //0 - 255 (255)

#define BOUNCE_TIME     5
#define VOLTAGE_ERROR  61


void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');     // send a capital X to indicate that Arduino is running
    delay(300);
  }
}


void setup() {

  digitalWrite(MOTOR_DIRECTION, 0);
  analogWrite(MOTOR_PWM, MAX_DUTYCYCLE);
  delay(1000);
  digitalWrite(MOTOR_DIRECTION, 1);
  analogWrite(MOTOR_PWM, 255 - MAX_DUTYCYCLE);
  delay(1000);
  digitalWrite(MOTOR_DIRECTION, 0);
  analogWrite(MOTOR_PWM, 0);
  delay(100);

  pinMode(MOTOR_DIRECTION, OUTPUT);
  pinMode(MOTOR_PWM, OUTPUT);

  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();  // send a byte to establish contact until receiver responds
}

void loop() {


  sensorStatus = analogRead(SENSOR);


  millisNow = millis();
  if(millisNow < millisStart){
   millisStart = millisNow;
  }

  stepTemp = -1;

  if(sensorStatus < VOLTAGE_ERROR){
    stepTemp = 1;
    if(millisNow - millisStart > BOUNCE_TIME){
      step = 1;
      millisNow = millis();
    }
  }

  if(sensorStatus < 367 + VOLTAGE_ERROR && sensorStatus > 367 - VOLTAGE_ERROR){
    stepTemp = 2;
    if(millisNow - millisStart > BOUNCE_TIME){
      step = 2;
      millisNow = millis();
    }
  }

  if(sensorStatus < 703 + VOLTAGE_ERROR && sensorStatus > 703 - VOLTAGE_ERROR){
    stepTemp = 3;
    if(millisNow - millisStart > BOUNCE_TIME){
      step = 3;
      millisNow = millis();
    }
  }
  if(stepTemp != stepTempOld){
    millisStart = millis();
    stepTempOld = stepTemp;
  }

  if(sensorStatus > 1023 - VOLTAGE_ERROR){
    stepTemp = 0;
    millisStart = millis();
  }

  if(step != stepOld){
    if(step == 1 && stepOld == 3){
      actualPoint++;
    }
    if(step == 2 && stepOld == 1){
      actualPoint++;
    }
    if(step == 3 && stepOld == 2){
      actualPoint++;
    }

    if(step == 1 && stepOld == 2){
      actualPoint--;
    }
    if(step == 2 && stepOld == 3){
      actualPoint--;
    }
    if(step == 3 && stepOld == 1){
      actualPoint--;
    }
    stepOld = step;
  }




  dutyCycle = (double)(abs(setPoint - actualPoint)) * (double)P_FRACTION;

  if(dutyCycle < MIN_DUTYCYCLE){
    dutyCycle = MIN_DUTYCYCLE;
  }
  if(dutyCycle > MAX_DUTYCYCLE){
    dutyCycle = MAX_DUTYCYCLE;
  }

  if(dutyCycle < MIN_DUTYCYCLE){
    dutyCycle = MIN_DUTYCYCLE;
  }
  if(dutyCycle > MAX_DUTYCYCLE){
    dutyCycle = MAX_DUTYCYCLE;
  }
  if(abs(setPoint - actualPoint) < STEP_MARGIN){
    analogWrite(MOTOR_PWM, 0);
    digitalWrite(MOTOR_DIRECTION, 0);
  }
  else{
    if(actualPoint < setPoint){
      digitalWrite(MOTOR_DIRECTION, 1);
      analogWrite(MOTOR_PWM, 255 - dutyCycle);
    }
    if(actualPoint > setPoint){
      digitalWrite(MOTOR_DIRECTION, 0);
      analogWrite(MOTOR_PWM, dutyCycle);
    }
  }



  readByte = 0;
  if (Serial.available() > 0){//if we get a valid byte, read from serial:
    //get incoming byte:
    readByte = Serial.read();
    Serial.print('r');   // send a 'r' to initiate next data from computer

    if(readByte > 0){
      if(readByte == 'h'){
        setPoint++;
      }
      if(readByte == 'g'){
        setPoint--;
      }
      if(readByte == 'f'){
        setPoint+=2875;
      }
      if(readByte == 'e'){
        setPoint-=2875;
      }
      if(readByte == 'd'){
        setPoint+=100;
      }
      if(readByte == 'c'){
        setPoint-=100;
      }
      if(readByte == 'b'){
        setPoint+=10;
      }
      if(readByte == 'a'){
        setPoint-=10;
      }
    }//if(readByte > 0){

  }//while (Serial.available() > 0){

}


